import { ChangeDetectionStrategy, ChangeDetectorRef, Component, HostBinding, HostListener, Input, TemplateRef, ViewChild } from "@angular/core";
import { debounceTime, distinctUntilChanged } from "rxjs/operators";
import { Subject } from "rxjs/Subject";
import { DataType } from "../data-operations/data-util";
import { BOOLEAN_FILTERS, DATE_FILTERS, NUMBER_FILTERS, STRING_FILTERS } from "../data-operations/filtering-condition";
import { IgxGridAPIService } from "./api.service";
var IgxGridFilterComponent = /** @class */ (function () {
    function IgxGridFilterComponent(gridAPI, cdr) {
        var _this = this;
        this.gridAPI = gridAPI;
        this.cdr = cdr;
        this.dialogShowing = false;
        this.UNARY_CONDITIONS = [
            "true", "false", "null", "notNull", "empty", "notEmpty",
            "yesterday", "today", "thisMonth", "lastMonth", "nextMonth",
            "thisYear", "lastYear", "nextYear"
        ];
        this.filterChanged = new Subject();
        this.filterChanged.pipe(debounceTime(250), distinctUntilChanged()).subscribe(function (value) { return _this.value = value; });
    }
    Object.defineProperty(IgxGridFilterComponent.prototype, "value", {
        get: function () {
            return this._value || "";
        },
        set: function (val) {
            if (!val) {
                this.clearFiltering();
                return;
            }
            this._value = this.transformValue(val);
            this.filter();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridFilterComponent.prototype, "dataType", {
        get: function () {
            return this.column.dataType;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridFilterComponent.prototype, "conditions", {
        get: function () {
            var conditions = [];
            switch (this.dataType) {
                case DataType.String:
                    conditions = Object.keys(STRING_FILTERS);
                    break;
                case DataType.Number:
                    conditions = Object.keys(NUMBER_FILTERS);
                    break;
                case DataType.Boolean:
                    conditions = Object.keys(BOOLEAN_FILTERS);
                    break;
                case DataType.Date:
                    conditions = Object.keys(DATE_FILTERS);
            }
            return conditions;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridFilterComponent.prototype, "template", {
        get: function () {
            switch (this.dataType) {
                case DataType.String:
                case DataType.Number:
                    return this.defaultFilterUI;
                case DataType.Date:
                    return this.defaultDateUI;
                case DataType.Boolean:
                    return null;
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridFilterComponent.prototype, "gridID", {
        get: function () {
            return this.column.gridID;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridFilterComponent.prototype, "styleClasses", {
        get: function () {
            return "igx-filtering";
        },
        enumerable: true,
        configurable: true
    });
    IgxGridFilterComponent.prototype.ngOnDestroy = function () {
        this.filterChanged.unsubscribe();
    };
    IgxGridFilterComponent.prototype.isActive = function (value) {
        return this._filterCondition === value;
    };
    Object.defineProperty(IgxGridFilterComponent.prototype, "unaryCondition", {
        get: function () {
            for (var _i = 0, _a = this.UNARY_CONDITIONS; _i < _a.length; _i++) {
                var each = _a[_i];
                if (this._filterCondition && this._filterCondition === each) {
                    return true;
                }
            }
            return false;
        },
        enumerable: true,
        configurable: true
    });
    IgxGridFilterComponent.prototype.filter = function () {
        this.gridAPI.filter(this.column.gridID, this.column.field, this._value, this.column.filteringCondition, this.column.filteringIgnoreCase);
        this.cdr.markForCheck();
    };
    IgxGridFilterComponent.prototype.clearFiltering = function () {
        this._value = null;
        this._filterCondition = undefined;
        this.gridAPI.clear_filter(this.gridID, this.column.field);
        this.cdr.markForCheck();
    };
    IgxGridFilterComponent.prototype.conditionChanged = function (value) {
        this._filterCondition = value;
        if (this.dataType === DataType.String) {
            this.column.filteringCondition = STRING_FILTERS[value];
        }
        else if (this.dataType === DataType.Number) {
            this.column.filteringCondition = NUMBER_FILTERS[value];
        }
        else if (this.dataType === DataType.Boolean) {
            this.column.filteringCondition = BOOLEAN_FILTERS[value];
        }
        else if (this.dataType === DataType.Date) {
            this.column.filteringCondition = DATE_FILTERS[value];
        }
        this.filter();
    };
    IgxGridFilterComponent.prototype.onInputChanged = function (val) {
        this.filterChanged.next(val);
    };
    IgxGridFilterComponent.prototype.toggle = function () {
        var _this = this;
        this.dialogShowing = !this.dialogShowing;
        if (this.dialogShowing) {
            var expr = this.gridAPI.get(this.gridID)
                .filteringExpressions.find(function (x) { return x.fieldName === _this.column.field; });
            if (expr) {
                this._value = expr.searchVal;
                this._filterCondition = expr.condition.name;
            }
        }
    };
    IgxGridFilterComponent.prototype.onClick = function (event) {
        event.stopPropagation();
    };
    IgxGridFilterComponent.prototype.transformValue = function (value) {
        if (this.dataType === DataType.Number) {
            value = parseInt(value, 10);
        }
        else if (this.dataType === DataType.Boolean) {
            value = Boolean(value);
        }
        else if (this.dataType === DataType.Date) {
            value = new Date(Date.parse(value));
        }
        return value;
    };
    IgxGridFilterComponent.decorators = [
        { type: Component, args: [{
                    changeDetection: ChangeDetectionStrategy.OnPush,
                    preserveWhitespaces: false,
                    selector: "igx-grid-filter",
                    styles: ["\n      .igx-filtering{position:static}.igx-filtering__toggle{z-index:10}.igx-filtering__toggle i{font-size:1em;line-height:1em;padding:.5em}.igx-filtering__toggle--active{background:#0375be !important;color:#fff}.igx-filtering__options{display:block;position:absolute;width:calc(100% - 32px);-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none;right:16px;left:16px;background:#fff;-webkit-box-shadow:0 2px 4px -1px rgba(0,0,0,0.26),0 4px 5px 0 rgba(0,0,0,0.12),0 1px 10px 0 rgba(0,0,0,0.08);box-shadow:0 2px 4px -1px rgba(0,0,0,0.26),0 4px 5px 0 rgba(0,0,0,0.12),0 1px 10px 0 rgba(0,0,0,0.08);border-radius:2px;padding:.5em;z-index:10;margin-top:8px}.igx-filtering__options>input{font-weight:initial;font-size:initial}\n    "],
                    template: "\n      <ng-template #defaultFilterUI>\n          <p *ngIf=\"!unaryCondition\">Value:\n              <input type=\"text\" igxInput autocomplete=\"off\" [value]=\"value\" (input)=\"onInputChanged($event.target.value)\" />\n          </p>\n      </ng-template>\n\n      <ng-template #defaultDateUI>\n          <igx-datePicker *ngIf=\"!unaryCondition\" [(ngModel)]=\"value\"></igx-datePicker>\n      </ng-template>\n\n      <div class=\"igx-filtering\">\n          <div class=\"igx-filtering__toggle\">\n              <span class=\"toggle-icon\" igxButton=\"icon\" [class.igx-filtering__toggle--active]=\"dialogShowing\" (click)=\"toggle()\">\n                  <igx-icon fontSet=\"material\" name=\"filter_list\"></igx-icon>\n              </span>\n          </div>\n      </div>\n\n      <span *ngIf=\"dialogShowing\" class=\"igx-filtering__options\">\n          <p>Condition:\n              <select (change)=\"conditionChanged($event.target.value)\">\n                  <option [selected]=\"isActive(each)\" *ngFor=\"let each of conditions\" [value]=\"each\">{{ each | filterCondition | titlecase }}</option>\n              </select>\n          </p>\n          <ng-container *ngTemplateOutlet=\"template; context: { $implicit: this }\"></ng-container>\n          <button igxButton igxRipple (click)=\"filter()\">Filter</button>\n          <button igxButton igxRipple (click)=\"clearFiltering()\">Reset</button>\n      </span>\n    "
                },] },
    ];
    /** @nocollapse */
    IgxGridFilterComponent.ctorParameters = function () { return [
        { type: IgxGridAPIService, },
        { type: ChangeDetectorRef, },
    ]; };
    IgxGridFilterComponent.propDecorators = {
        "column": [{ type: Input },],
        "styleClasses": [{ type: HostBinding, args: ["class",] },],
        "defaultFilterUI": [{ type: ViewChild, args: ["defaultFilterUI", { read: TemplateRef },] },],
        "defaultDateUI": [{ type: ViewChild, args: ["defaultDateUI", { read: TemplateRef },] },],
        "onClick": [{ type: HostListener, args: ["click", ["$event"],] },],
    };
    return IgxGridFilterComponent;
}());
export { IgxGridFilterComponent };
//# sourceMappingURL=grid-filtering.component.js.map